#include <stdio.h>
#include <string.h>
//
// AnalogInput_InternalThermometer_430
// Analog Input - MSP430 Internal Thermometer
//
// Demonstrates analog input by reading the internal temperature sensor.
//
// Created by Robert Wessels
// modified 7 March 2012
// By Robert Wessels
//
// This example code is in the public domain.
//
// Revision:
// Rei VILO, Mar 12, 2012 - One decimal place
// Rei VILO, Mar 13, 2012 - More precise algorithm
// Rei VILO, Mar 14, 2012 - Average
// Rei VILO, May 21, 2012 - Updated with GREEN_LED, RED_LED and PUSH2
// Rei VILO, Sep 09, 2012 - TimerSerial.h no longer required
// Rei VILO, Sep 24, 2012 - FraunchPad excluded
// Brian Cruickshank, July 19, 2017 - Serial I/O interface to GCv2

// Tested on msp430g2452 and msp430g2553
#if defined(__MSP430G2452__) || defined(__MSP430G2553__) || defined(__MSP430G2231__)
#else
#error Board not supported
#endif
// RED_LED, GREEN_LED, TEMPSENSOR, PUSH2 already defined
#define NUMBER 200
#define MAX_JSON_STRLEN 40
#define NUM_VALUES 10
int greenLedState = HIGH;
int redLedState = LOW;
uint8_t i = 0;
uint32_t average = 0;
uint32_t values[NUM_VALUES];
char jsonStrFromHost[MAX_JSON_STRLEN];
char badStrFromHost[MAX_JSON_STRLEN];
#define FARENHEIT 0
#define CELSIUS 1
uint8_t numBytesRead = 0;
uint8_t j = 0;
uint8_t n = 0;
int rxStrIndex = 0;
int badStrIndex = 0;
int numBadStrings = 0;
boolean toggle = false;
boolean cmdReceived = false;
boolean flag = true;
boolean running = true;
boolean msgOk = false;
int debounceCounter = 0;
void setup() {
    pinMode(RED_LED, OUTPUT);
    pinMode(GREEN_LED, OUTPUT);
    analogReference(INTERNAL1V5);
    analogRead(TEMPSENSOR); // first reading usually wrong
    Serial.setTimeout(6000); // 1s timeout when reading from serial port
    Serial.begin(9600);
    pinMode(PUSH2, INPUT_PULLUP);
    greenLedState = HIGH;
    redLedState = LOW;
    digitalWrite(GREEN_LED, greenLedState);
    digitalWrite(RED_LED, redLedState);
    average = 0;

    for (j=0; j<NUM_VALUES; j++) {
        values[j]= getTemperatureTimes100(FARENHEIT);
        average += values[j];
    }
    clearStr(jsonStrFromHost);
    clearStr(badStrFromHost);
    rxStrIndex = 0;

    Serial.flush();
    j=0;
}
void printDec(uint32_t ui) {
    if (ui < 9999){
        Serial.print(ui/100, DEC);
        Serial.print(".");
        if ((ui%100) < 10){
            Serial.print("0");
        }
        Serial.print(ui%100, DEC);
    }
    else {
        Serial.print("99.99");
    }
}
// For debugging purposes: if you get bad strings from the host,
// the last bad string will be copied into the badStrFromHost character array
void copyBadStr(){
    int m = 0;
    clearStr(badStrFromHost);

    for (m=0; m < MAX_JSON_STRLEN; m++){
        badStrFromHost[m] = jsonStrFromHost[m];
    }
    badStrIndex = rxStrIndex;
    rxStrIndex = 0;
}

void clearStr(char* pBuffer){
    int k = 0;
    for (k=0; k < MAX_JSON_STRLEN; k++){
        pBuffer[k] = (char)0;
    }
}
// Formula: http://www.43oh.com/forum/viewtopic.php?p=18248#p18248
uint32_t getTemperatureTimes100(int scale){
    if (scale == FARENHEIT) {
        return ((uint32_t)analogRead(TEMPSENSOR)*48724L - 30634388L) *100 >> 16;
    } else {
        // Celsius
        return ((uint32_t)analogRead(TEMPSENSOR)*27069 - 18169625) *100 >> 16;
    }
}

int getSerialString(){
    char rxChar;
    boolean done = false;
    int waitCtr = 0;
    int numCharsRx = 0;
    int numAvailable = 0;
    int index = 0;
    while (!done) {
        numAvailable = Serial.available();
        if (numAvailable > 0) {
            for (index=0; index < numAvailable; index++){
                waitCtr = 0;
                rxChar = Serial.read();
                if ((rxChar != -1) && (rxChar != (char)0)){
                    if (rxChar == (char)10){
                        done = true;
                        numCharsRx = rxStrIndex;
                        rxStrIndex = 0;
                    } else {
                        if (rxStrIndex < MAX_JSON_STRLEN){
                            jsonStrFromHost[rxStrIndex++] = rxChar;
                        } else {
                            done = true;
                            break;
                        }
                    }
                }
            }
            Serial.flush();
        } else {
            if (rxStrIndex == 0) {
                done = true;
                Serial.flush();
            } else {
                // If a partial command has been received
                // keep waiting until the /n delimiter is received
                // or we timeout
                delayMicroseconds(50);     // pauses for 0.05 milliseconds
                waitCtr++;
                if (waitCtr > 2000) {  // timeout after 1 second
                    clearStr(jsonStrFromHost);
                    rxStrIndex = 0;
                    numCharsRx = 0;
                  done = true;
                  Serial.flush();
                }
              }
        }
    }
    return(numCharsRx);

}  
void loop() {
    if (running){


        numBytesRead = getSerialString();//Serial.readBytesUntil('\n',jsonStrFromHost,MAX_JSON_STRLEN);
        if (numBytesRead > 2){
            // Data from the host needs to be of the form {"LEDs":{"Red":0,"Green":0}} to turn off
            // the red and green LEDs, {"LEDs":{"Red":1,"Green":1}} to turn on the red and green LEDs.
            // (Code is not generic at all, but it is small  )
            msgOk = false;

            for (int j=2; j < numBytesRead; j++){
                if ((jsonStrFromHost[j] == ':') && (j < (numBytesRead + 1))){
                    if (jsonStrFromHost[j-2] == 'd'){
                        msgOk = true;
                        if (jsonStrFromHost[j+1] == '1'){
                            redLedState = HIGH;
                        }  else {
                            redLedState = LOW;
                        }
                        digitalWrite(RED_LED, redLedState);
                        cmdReceived = true;
                        flag=true;
                    } else {
                        if (jsonStrFromHost[j-2] == 'n'){
                            msgOk = true;
                            if (jsonStrFromHost[j+1] == '1'){
                                greenLedState = HIGH;
                            }  else {
                                greenLedState = LOW;
                            }
                            digitalWrite(GREEN_LED, greenLedState);
                            cmdReceived = true;
                            flag=true;
                        }
                    }
                }
            }
            if (!msgOk){
                // put a breakpoint here
                numBadStrings++;
                copyBadStr();
            } else {
                clearStr(jsonStrFromHost);
                rxStrIndex = 0;
                flag = true;
            }

        } else {
            if (numBytesRead > 0) {
                numBadStrings++;
                copyBadStr();
            }
        }


        average -= values[n%NUM_VALUES];
        values[n%NUM_VALUES] = getTemperatureTimes100(FARENHEIT);
        average += values[n%NUM_VALUES];
        // Print average as a JSON string of the form {"temp":99.99,"LEDs":{"Red":0,"Green":1}}\n
        if (flag) {
            Serial.print("{\"temp\":");
            printDec(average/NUM_VALUES);
            Serial.print(",\"LEDs\":{\"Red\":");
            Serial.print(redLedState, DEC);
            Serial.print(",\"Green\":");
            Serial.print(greenLedState, DEC);
            Serial.print("}}\n");
            flag = false;
            n = 0;
        }

    } else {
        // if not running, blink the red LED.
        if (flag) {
            digitalWrite(RED_LED, toggle);
            toggle = !toggle;
            flag = false;
            n = 0;
        }
    }
    n++;
    if (n==NUMBER) {
       flag=true;
       n = 0;
    }
    if (digitalRead(PUSH2)==LOW) {
        if (debounceCounter == 2){
            running = !running;
            if (running) {
                greenLedState = HIGH;
                redLedState = LOW;
            } else {
                greenLedState = LOW;
                redLedState = LOW;
            }
            digitalWrite(GREEN_LED, greenLedState);
            digitalWrite(RED_LED, redLedState);
            debounceCounter = 9;
        } else {
            debounceCounter++;
        }
    } else {
        debounceCounter = 0;
    }

    delay(1);
}



